//---------------------------------------------------------------------
//	File:		dspicservo.h
//
//	Written By:	Lawrence Glaister VE7IT
//
// Purpose: Various header info for the project
//      
// 
//---------------------------------------------------------------------
//
// Revision History
// March 11 2006 --   formatted into multi file project
// Sept 25 2006 -  6x pwm rate
//---------------------------------------------------------------------- 
// define which chip we are using (peripherals change)
#include <p30f4012.h>

// Core frequency = number of IPS
#define FCY  29480000       // FRC+PLL

/* define required pwm rate... at 30KHz good to have a nice fast +-10V output 
without loosing resolution */
#define FPWM 30E3

// define some i/o bits for the various modules
#define IN1				_RB0
#define OUT1			_LATE0
#define SVO_ENABLE		_RB1
#define ErrLED			_LATB2
#define FltPin			_RE8

#define setErrLed()	(ErrLED = 1)
#define clearErrLed()	(ErrLED = 0)
#define toggleErrLed()	(ErrLED = !ErrLED)

// for some reason PI may not be defined in math.h on some systems
#ifndef M_PI
#define M_PI (3.14159265358979323846)
#endif

#define	TRUE	(1)
#define	FALSE	(0)	

struct PID{
    long int command;	/* commanded value */
    long int feedback;	/* feedback value */
    float error;		/* command - feedback */
    float deadband;		/* param: deadband */
    float maxerror;		/* param: limit for error */
    float maxerror_i;	/* param: limit for integrated error */
    float maxerror_d;	/* param: limit for differentiated error */
    float maxcmd_d;		/* param: limit for differentiated cmd */
    float error_i;		/* opt. param: integrated error */
    float prev_error;	/* previous error for differentiator */
    float error_d;		/* opt. param: differentiated error */
    long int prev_cmd;	/* previous command for differentiator */
    float cmd_d;		/* opt. param: differentiated command */
    float bias;			/* param: steady state offset */
    float pgain;		/* param: proportional gain */
    float igain;		/* param: integral gain */
    float dgain;		/* param: derivative gain */
    float ff0gain;		/* param: feedforward proportional */
    float ff1gain;		/* param: feedforward derivative */
    float maxoutput;	/* param: limit for PID output */
    float output;		/* the output value */
    short enable;		/* enable input */
    short limit_state;	/* 1 if in limit, else 0 */
	short multiplier;	/* pc command multiplier */
	short ticksperservo; /* number of 100us ticks/servo cycle */
    int cksum;		/* data block cksum used to verify eeprom */
};

#define TEST_MAXSAMPLES 200
struct TESTSTRUCT{
	int testInRun;
	int ticks;
	int prediv;				// sampleing frequency = 10KHz / (prediv+1)
	int printCurve;
	int sampleToPrintIndex;
	int sampleIndex;
	int samples[TEST_MAXSAMPLES];
};
extern struct TESTSTRUCT testS;
